@@ -16,10 +16,11 @@
 #    along with this program; if not, write to the Free Software
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
-import os.path, os, re, time, socket
+import socket
+import time
+import re
+import os
 from Base import BaseFolder
-from offlineimap import imaputil
-from offlineimap.ui import UIBase
 from threading import Lock
 
 try:
@@ -61,6 +62,7 @@ class MaildirFolder(BaseFolder):
         self.repository = repository
         self.accountname = accountname
         BaseFolder.__init__(self)
+        #self.ui is set in BaseFolder.init()
 
     def getaccountname(self):
         return self.accountname
@@ -160,16 +162,17 @@ class MaildirFolder(BaseFolder):
         return retval
 
     def quickchanged(self, statusfolder):
+        """Returns True if the Maildir has changed"""
         self.cachemessagelist()
-        savedmessages = statusfolder.getmessagelist()
-        if len(self.messagelist) != len(savedmessages):
+        # Folder has different uids than statusfolder => TRUE
+        if sorted(self.getmessageuidlist()) != \
+                sorted(statusfolder.getmessageuidlist()):
             return True
-        for uid in self.messagelist.keys():
-            if uid not in savedmessages:
+        # Also check for flag changes, it's quick on a Maildir 
+        for (uid, message) in self.getmessagelist().iteritems():
+            if message['flags'] != statusfolder.getmessageflags(uid):
                 return True
-            if self.messagelist[uid]['flags'] != savedmessages[uid]['flags']:
-                return True
-        return False
+        return False  #Nope, nothing changed
 
     def cachemessagelist(self):
         if self.messagelist is None:
@@ -193,8 +196,7 @@ class MaildirFolder(BaseFolder):
     def savemessage(self, uid, content, flags, rtime):
         # This function only ever saves to tmp/,
         # but it calls savemessageflags() to actually save to cur/ or new/.
-        ui = UIBase.getglobalui()
-        ui.debug('maildir', 'savemessage: called to write with flags %s and content %s' % \
+        self.ui.debug('maildir', 'savemessage: called to write with flags %s and content %s' % \
                  (repr(flags), repr(content)))
         if uid < 0:
             # We cannot assign a new uid.
@@ -226,7 +228,7 @@ class MaildirFolder(BaseFolder):
             else:
                 break
         tmpmessagename = messagename.split(',')[0]
-        ui.debug('maildir', 'savemessage: using temporary name %s' % tmpmessagename)
+        self.ui.debug('maildir', 'savemessage: using temporary name %s' % tmpmessagename)
         file = open(os.path.join(tmpdir, tmpmessagename), "wt")
         file.write(content)
 
@@ -238,7 +240,7 @@ class MaildirFolder(BaseFolder):
         file.close()
         if rtime != None:
             os.utime(os.path.join(tmpdir,tmpmessagename), (rtime,rtime))
-        ui.debug('maildir', 'savemessage: moving from %s to %s' % \
+        self.ui.debug('maildir', 'savemessage: moving from %s to %s' % \
                  (tmpmessagename, messagename))
         if tmpmessagename != messagename: # then rename it
             os.rename(os.path.join(tmpdir, tmpmessagename),
@@ -256,7 +258,7 @@ class MaildirFolder(BaseFolder):
         self.messagelist[uid] = {'uid': uid, 'flags': [],
                                  'filename': os.path.join(tmpdir, messagename)}
         self.savemessageflags(uid, flags)
-        ui.debug('maildir', 'savemessage: returning uid %d' % uid)
+        self.ui.debug('maildir', 'savemessage: returning uid %d' % uid)
         return uid
         
     def getmessageflags(self, uid):
