@@ -17,9 +17,8 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
 
 from Base import BaseRepository
-from offlineimap import folder, imaputil
-from offlineimap.ui import UIBase
-from mailbox import Maildir
+from offlineimap import folder
+from offlineimap.ui import getglobalui
 import os
 from stat import *
 
@@ -31,7 +30,7 @@ class MaildirRepository(BaseRepository):
 
         self.root = self.getlocalroot()
         self.folders = None
-        self.ui = UIBase.getglobalui()
+        self.ui = getglobalui()
         self.debug("MaildirRepository initialized, sep is " + repr(self.getsep()))
 	self.folder_atimes = []
 
